//*****************************************************************************
//      C++ functions for:
//              Program control functions (including libsx callbacks)
//
//      Written by Stuart Prescott:
//              Written:                21/08/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//		character typing
//		standard library
//
//      Other Object Classes used:
//              orbital selector
//              colours
//              fonts
//              main program data structure
//              orbital drawing
//              information window
//              interpolation settings window
//		callback
//
//*****************************************************************************

extern "C" { 
    #include <libsx.h>
}

#include <stdlib.h>
#include <ctype.h>

#include <iostream.h>

#include "selecto.h"
#include "main.h"
#include "callback.h"
#include "h3d.h"
#include "intwin.h"
#include "info.h"

#include "fontdefs.h"		//font initialisations
#include "coldefs.h"	        // the colour initialisations
  
extern ColourDefs *pal;
extern FontDefs *fonts;


#define MONITORREDISPLAY 0

// *************************************************************************
// *************************************************************************
void
Redisplay(Widget w, int width, int height, void *data)
{
    // Redisplay function for the main draw_area.
    // acts according to the current mode of the program,
    // NOTE: this may be used as other than a callback, to allow refreshing
    // of the main draw_area.

    MyProgram *me = (MyProgram *)data;

    me->mainwin.height = height;
    me->mainwin.width  = width;

    #if MONITORREDISPLAY
	cout << "===Redisplay\n";
    #endif

    switch (me->mode)
    {
	case WAITING_FOR_SELECTION:
	{
	    //*** Draw the Icon Boxes then wait for a selection
	    //*** this mode is only left once a selction is made . . .
	    me->mainwin.orbitalselect.Setup(me->mainwin.draw_area);
	
	    break;
	}
	
	case DRAWING_ORBITAL_DENSITY:
	{
	    //*** Do the rendering . . .
	    me->mainwin.orbital.Show(me->mainwin.orbital.fntype,
				     width, height);
	    break;
	}
	
	default:
	{
	    //** it should never get here
	    cerr << "*******UNDEFINED mode"<<me->mode<<"\n";
	    char dummy; cin>>dummy;

	    break;
	}
    }
    SetDrawArea   (me->mainwin.title_area);
    TitleRedisplay(me->mainwin.title_area, 1,1, me->mainwin.orbital.view);

    return; 
}

// *************************************************************************
// *************************************************************************
void Button_functype(Widget w, void *data)
{
    // CB to change whether the WF or DENSITY is being plotted

    MyProgram *me = (MyProgram *)data;
    me->mainwin.orbital.fntype= (me->mainwin.orbital.fntype%2) + 1;

    SetDrawArea(me->mainwin.draw_area);
    Redisplay(me->mainwin.draw_area, me->mainwin.width, me->mainwin.height, me);

    return;		//having forced redisplay
}

// *************************************************************************
// *************************************************************************
void Button_SetMode(Widget w, void *data)
{
    // CB to select a new orbital

    MyProgram *me = (MyProgram *)data;

    me->mode = WAITING_FOR_SELECTION;

    SetDrawArea(me->mainwin.draw_area);
    Redisplay(w,  me->mainwin.width, me->mainwin.height, me);

    return;             //having forced redisplay
}

// *************************************************************************
// *************************************************************************
void Button_ResetView(Widget w, void *data)
{
    // CB to restore view back to vector(1,1,1);

    MyProgram *me = (MyProgram *)data;

    me->arcballwin->ResetMatrix();
    
    SetDrawArea(me->mainwin.draw_area);
    Redisplay(w,  me->mainwin.width, me->mainwin.height, me);

    return;             //having forced redisplay
}

// *************************************************************************
// *************************************************************************
void
Key_Quit_Func(Widget w, char *key, int up_or_down, void *data)
{
    // Check to see if the key press is a "q". If so, quit.

    if ( key[0] == 'q' )
    {
	CleanExit(0);
    }

    return; 
}

// *************************************************************************
// *************************************************************************
void
Button_quit(Widget w, void *data)
{
    // CB to quit the program

    CleanExit(0);

    // function never returns  . . .
}

// *************************************************************************
// *************************************************************************
void
NewHelpWin(MyProgram *me)
{
    // CB to launch the information window

    me->infowin = new InfoWin;
    return;
}

// *************************************************************************
// *************************************************************************
void ColourError()
{
    // func for colourmap problems . . .

    cerr << "Could not get colour map!!!";
    char dummy='\0';
    while ((dummy!='y')&&(dummy!='n'))
    {
        cerr << "\nProceed with program (y/n)? ";
        cin >> dummy;
    }
    if (dummy=='n') CleanExit(-1);
    return;
}

// *************************************************************************
// *************************************************************************
void CleanExit(int i)
{
    // Close any open windows . . .
    SetCurrentWindow(ORIGINAL_WINDOW);
    CloseWindow();
    //SyncDisplay();

    //delete pal;
    //delete fonts;

    exit (i);
}

