//*****************************************************************************
//      C++ functions for:
//		graph class
//
//      Written by Stuart Prescott:
//              Written:                20/09/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//              Formatted I/O
//              string
//
//      Other Object Classes used:
//              template arrays
//              fonts
//              maths extensions
//
//*****************************************************************************
  
extern "C"  
{  
    #include <libsx.h>  
}  
  
#include <stdio.h>  
#include <iostream.h>  
#include <string.h>  

#include "graph.h"
#include "arraytem.h"
#include "mathxtra.h"  
#include "fontdefs.h"
  
// default states are 0  
#define DEBUGAXES	0  
#define DEBUGCURVE	0  
#define DEBUGFONTS	0  
#define DEBUGMARKLEVEL	0  
  
// default states are 1!!!!  
#define GRAPHSAFETY 1  
#define GETFONTINFO 1  
#define NOSKIP	    1  
  
extern FontDefs *fonts;

//*****************************************************************************
//*****************************************************************************
void Graph::drawaxes ()  
{  
    int    i,j,k,xllen,yllen,tlen,ext,ticszx,ticszy;  
    double v,delv;  
    char str[10];  
  
    #if GETFONTINFO  
	XFont font;  
  
	#if DEBUGFONTS  
	    cout << "DrawArea in drawaxes:"<< draw_area <<"\n";  
	#endif  
	font = fonts->plotscale; 
	SetWidgetFont(draw_area, font); 
	int fonth=FontHeight(font);  
    #else  
	#define fonth 10  
    #endif  
  
    #if GRAPHSAFETY  
        open=TRUE;  
    #endif  
  
    #if NOSKIP  
	xllen= strlen (xlab);  
	yllen= strlen (ylab);  
	tlen= strlen (title);  
    #endif  
  
    //**** generate plot margins, borders, and scaling factors	****  
    //**** p1= lower left border, p2= upper right border	****  
    //**** orig= axes lower left, end= axes upper right		****  
    //**** min, max are user axes range, psc is user scale	****  
  
    xorig = xp1 + 1 + 3*fonth;  
    yorig = yp1 - 1 - fonth;  
  
  
    ticszx= 0;  
    if (xtic > 0)  
    {  
	ticszx= 1;  
	if (ypix > 200) ticszx= 2;  
        yorig -= ticszx ;  
	if (xllen > 0)  yorig -= (1 + fonth);  
    }  
    ticszy= 0;  
    if (ytic > 0)  
    {  
	ticszy= 1;  
	if (ypix > 200) ticszy++;  
	xorig += ticszy;  
	if (yllen > 0)  xorig += 1 + fonth;  
    }  
    xend = xorig + xaxlen;  
    yend = yorig - yaxlen;  
    xp2  = xend + 1 + fonth;  
    yp2 = yend - 1 - fonth;  

    #if DEBUGAXES  
        if ( (xp2 > xpix) || (yp2 < 0) || (xp1 < 0) || (yp1 >= ypix) )  
        {  
            printf("AXES ERROR: range= %d %d to %d %d\n",xp1,yp1,xp2,yp2);  
            char k;  
            cin >> k;  
        }  
    #endif  
  
    if (xmin == xmax)  
    {  
	xmin = 0.0;  
	xmax = xaxlen;  
	xpsc = 1.0;  
    }  
    else  
    {  
	xpsc = xaxlen / (xmax-xmin);  
    }  
  
    if (ymin == ymax)  
    {  
	ymin = 0.0;  
	ymax = yaxlen;  
	ypsc = 1.0;  
    }  
    else  
    {  
	ypsc = yaxlen / (ymax-ymin);  
    }  
  
    SetColor (col);  
  
    //**** draw axes ****  
    if (bdr == 1) DrawBox (xorig, yorig, xaxlen, -yaxlen);  
    //**** draw x tic marks and scale ****  
    if (xtic > 0)  
    {  
	delv= (xmax-xmin) / xtic;  
	for (v= xmin; v<=xmax; v+=delv)  
        {  
            i= xorig + INT( xpsc * (v-xmin) );  
            for (k= 1; k<=ticszx; ++k)  
            {  
                DrawPixel (i,yorig + k);  
                DrawPixel (i,yend - k);  
            }  
	    #if DEBUGAXES  
		cout << "X-axis\n";  
	    #endif  
            //draw x scale
	    #if NOSKIP  
		ext = sprintf( str, "%.f", v);  
        	ext = TextWidth(font, str);  
        	DrawText (str, i-ext/2 , yorig+1+fonth);  
	    #endif  
	}  
    }  
  
    //**** draw y tic marks and scale ****  
    if (ytic > 0)  
    {  
	delv= (ymax-ymin) / ytic;  
	for (v= ymin; v<=ymax; v+=delv)  
        {  
            i= yorig - INT( ypsc * (v-ymin) );  
            for (k= 1; k<=ticszy; ++k)  
            {  
                DrawPixel (xorig-k,i);  
                DrawPixel (xend+k,i);  
            }  
	    #if NOSKIP  
		//draw the y scale
		ext = sprintf( str, "%g", v);  
        	ext = TextWidth(font, str);  
        	DrawText (str, xorig-ext-ticszy, i+fonth/2);  
	    #endif  
        }  
    }  
  
    #if NOSKIP  
	//**** draw x axis label ****  
	if (xllen > 0)  
	{  
	    ext = TextWidth(font, xlab);  
	    j= yorig + 1 + ticszx + fonth;  
	    if (xtic > 0) j += fonth;  
            DrawText (xlab, (xorig+xend-ext)/2, j);  
	}  
  
	//**** draw y axis label ****  
	if (yllen > 0)  
	{  
	    ext= int( (yend+yorig)/2 );// + (fonth * (yllen))/2);  
            j= xorig-4*fonth - 1 - ticszy;  
            char s[2];  
	    int halflen=int(yllen/2);  
            s[1]= '\0';  
            for (i=0; i<yllen; ++i)  
            {  
        	s[0]=ylab[i];  
        	DrawText(s, j, ext +fonth*(i-halflen));  
            }  
	}  
  
	//**** draw title ****  
	if (tlen > 0)  
	{  
	    j= yend - 3;  
	    font = fonts->plottitle;
	    SetWidgetFont(draw_area, font);
	    fonth=FontHeight(font);
	    ext = TextWidth(font, title);  
            DrawText (title, (xorig+xend-ext)/2, j);  
	}  
    #endif  
  
    return;  
}  
  
//*****************************************************************************
//*****************************************************************************
void Graph::eraseaxes ()  
{  
    SetColor(bcol);  
    DrawFilledBox (xp1,yp1, xp2-xp1, yp2-yp1);  
    SetColor(col);  
    return;  
}  
  
//*****************************************************************************
//*****************************************************************************
void Graph::curve (const doublearray1& x, const doublearray1& y, int fgcol)  
{  
    //*** draws a curve (where the x and y values are in the two arrays . . .
    // fgcol specifies the colour to be used for the graph (a value of -1
    // indicates to use the current graph colour)
    //
    // the colour used should be the same as the axes (although this is not 
    // necessary) because any points outside the axis bounds will be truncated
    // to the axes.

    if (fgcol==-1) fgcol=col;  
  
    #if DEBUGCURVE  
	cout <<"Drawing Curve . . .\n";  
    #endif  
    
    // it is necessary that the axes be previously drawn to initialise the
    // pixel/real ratios . . . 
    #if GRAPHSAFETY  
	//cout << "check for graph open\n";  
        if (!open)  
        {  
            cerr << "Plot "<< this << " not open for curve drawing\n";  
            return;  
        }  
    #endif  
  
    int npts = (x.max()-x.min() +1);  
    XPoint *pts;  
    pts = new XPoint[npts];  
  
    double xval, yval;  
    int xarraymin=x.min();  
    int yarraymin=y.min();  
   
    #if DEBUGCURVE  
	printf ("\nxarraymin= %d yarraymin= %d\n", xarraymin, yarraymin);  
        printf ("xaxlen= %d yaxlen= %d\n", xaxlen, yaxlen);  
        printf ("xpsc= %f ypsc= %f\n", xpsc, ypsc);  
	printf ("xorig=%d yorig=%d\n", xorig, yorig);  
        printf ("xmax=%f ymax=%f\n", xmax, ymax);  
	printf ("xmin=%f ymin=%f\n", xmin, ymin);  
    #endif  
   
  
    #if GRAPHSAFETY  
    if (xarraymin != yarraymin)  
    {  
        cerr << "x and y have different array bounds: "  
             << xarraymin <<" "<< yarraymin <<"\n";  
    }  
    if ((ypsc==0) || (xpsc==0))
    {
         cerr << "xpsc or ypsc == 0 "  
             <<xpsc <<" "<< ypsc <<"\n";  
    }
    #endif  
  
    for (int i=0; i<npts; ++i)  
    {  
	xval=MIN(xmax, x.get(i+xarraymin));  
        xval=MAX(xmin, xval);   
        yval=MIN(ymax, y.get(i+yarraymin));        
        yval=MAX(ymin, yval);   
	//xval= x.get(i+xarraymin);  
	//yval=  y.get(i+yarraymin);   
	pts[i].x = xorig + INT( (xval - xmin) * xpsc);  
	pts[i].y = yorig - INT( (yval - ymin) * ypsc);	  
	#if DEBUGCURVE	  
	    printf("%d  %f\t%f\t%d\t%f\t%f\t%d\n",   
	            i, x.get(i+xarraymin), xval, pts[i].x,  
	               y.get(i+yarraymin), yval, pts[i].y  );  
   	#endif   
    }  
    SetColor(fgcol);  
    DrawPolyline (pts, npts);  
    delete[] pts;  
    return;  
}  
  
//*****************************************************************************
//*****************************************************************************
  
void Graph::marklevel (double val, int fgcol)  
{  
    // marks a level (using a horz line) on the graph.
    // a colour of -1 indicates to use the current graph colour

    #if DEBUGMARKLEVEL  
	printf ("given %d;", fgcol);  
    #endif  
  
    //**** this draws a level marking on 1D plots ****  
    if (fgcol==-1) fgcol=col;  
  
    #if DEBUGMARKLEVEL  
	printf (" using %d\n", fgcol);  
    #endif  
      
    doublearray1 xx(1,2), yy(1,2);  
    //**** +ve level ****  
    xx.set(1, xmin);  
    xx.set(2, xmax);  
    yy.set(1, val);  
    yy.set(2, val);  
  
    curve (xx,yy, fgcol);  
  
    return;  
}  
  
  
  
