//*****************************************************************************
//      C++ functions for:
//		Orbital Selction Procedure.
//
//  		Original subroutine written in FORTRAN by
//      	Jeff Reimers and George Bacskay,
//		(School of Chemistry, University of Sydney)
//
//      Written/Adapted by Stuart Prescott:
//              Written:                07/10/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Formatted I/O
//
//      Other Object Classes used:
//              template arrays
//              fonts
//              libsx-extensions
//              main program data structure
//              orbital id and name
//		callbacks
//
//*****************************************************************************

extern "C"  
{  
    #include <libsx.h>  
}  

#include <stdio.h>
  
#include "selecto.h"
#include "main.h"		//declares the types . . .  
#include "arraytem.h"		//other prototypes required  
#include "callback.h"  
#include "grafxten.h"  
#include "coldefs.h"  
#include "fontdefs.h"
  
#define DEBUGSELECT 0  
#define DEBUGSELECTSTR 0  
#define DEBUGDRAWBOX 0  
#define DEBUGNLMCALLBACK 0  
#define DEBUGMARKCELL 0  
  
extern ColourDefs *pal;
extern FontDefs   *fonts;
  
OrbitalSelect::OrbitalSelect()  
{  
    cwid    = new intarray1(-5,  5);            //icon width  
    csta    = new intarray1(-5,  5);            //column start positions  
    x1      = new intarray1( 1, 92);            //icon boundaries       L  
    x2      = new intarray1( 1, 92);            //                      R  
    y1      = new intarray1( 1, 92);            //                      T  
    y2      = new intarray1( 1, 92);            //                      B  
    deltax  = new intarray1( 1, 92);            //                      R-L  
    deltay  = new intarray1( 1, 92);            //                      B-T  
    qn      = new intarray1( 1, 92);  
    ql      = new intarray1( 1, 92);  
    qm      = new intarray1( 1, 92);  
  
    cwid->set(-5, 84);  
    cwid->set(-4, 86);  
    cwid->set(-3, 67);  
    cwid->set(-2, 56);  
    cwid->set(-1, 23);  
    cwid->set( 0, 18);  
    cwid->set( 1, 23);  
    cwid->set( 2, 28);  
    cwid->set( 3, 67);  
    cwid->set( 4, 66);  
    cwid->set( 5, 90);  
  
    //**** column start positions ****  
    int i, j= 0;  
    for (i= -5; i<=5; ++i)  
    {  
	csta->set(i, j);  
	j+= cwid->get(i) + 3;  
    }  
  
  
    int miwf;  
    yspace= 21;  
    int y= yspace;  
    int iwf= 0;  
    //**** form grid ****  
    for (n= 1; n<=6; ++n)  
    {  
    	miwf= 0;  
	for (l= 0; l<=n-1; ++l)  
        {  
            for (m= -l; m<=l; ++m)  
            {  
                iwf++;  
                miwf++;  
                qn->set(iwf, n);  
                ql->set(iwf, l);  
                qm->set(iwf, m);  
                x1->set(iwf, csta->get(m));  
                deltax->set(iwf, cwid->get(m) + 1);  
                x2->set(iwf, x1->get(iwf) + deltax->get(iwf));  
                y1->set(iwf, y + 1);  
                deltay->set(iwf, yspace - 1);  
                y2->set(iwf, y + deltay->get(iwf));  
            }  
            y+= yspace;  
	}  
    }  
  
    //orbital names are fixed . . . hard wire them in!  
    lwf[0]= new char[5];  
    lwf[1]=  "s";  
  
    lwf[2]=  "px";  
    lwf[3]=  "pz";  
    lwf[4]=  "py";  
  
    lwf[5]=  "dx2-y2";  
    lwf[6]=  "dxz";  
    lwf[7]=  "dz2";  
    lwf[8]=  "dyz";  
    lwf[9]=  "dxy";  
  
    lwf[10]= "fy3-3x2y";  
    lwf[11]= "f(x2-y2)z";  
    lwf[12]= "fxz2";  
    lwf[13]= "fz3";  
    lwf[14]= "fyz2";  
    lwf[15]= "fxyz";  
    lwf[16]= "f3xy2-x3";  
  
    lwf[17]= "gy4-6x2y2+x4";  
    lwf[18]= "g(y3-3xy2)z";  
    lwf[19]= "g(x3-y3)z2";  
    lwf[20]= "gxz3";  
    lwf[21]= "gz4";  
    lwf[22]= "gyz3";  
    lwf[23]= "gxyz2";  
    lwf[24]= "g(3xy2-x3)z";  
    lwf[25]= "gxy3-x3y";  
  
    lwf[26]= "h5xy4-10x3y+x3";  
    lwf[27]= "h(y4-6x2y2+x4)z";  
    lwf[28]= "h(y3-3xy2)z2";  
    lwf[29]= "h(x3-y3)z3";  
    lwf[30]= "hxz4";  
    lwf[31]= "hz5";  
    lwf[32]= "hyz4";  
    lwf[33]= "hxyz3";  
    lwf[34]= "h(3xy2-x3)z2";  
    lwf[35]= "h(xy3-x3y)z";  
    lwf[36]= "hy5-10x2y3+5x4y";  
}  
  
void OrbitalSelect::Setup(Widget w)  
{  
    #if DEBUGSELECT  
	cout << "Will Draw Boxes . . .\n";  
    #endif  
  
    draw_area = w;  
    SetBgColor(w, pal->black);
    ClearDrawArea();  
    SetColor(pal->white);
    DrawIcons();  
    SetButtonDownCB(draw_area, nlmButtonDown);  
  
    #if DEBUGSELECT  
	cout << "Set MouseDown CB\n";  
    #endif  
  
    //now go to mainloop to wait for the selection . . .  
    return;
}  
  
// *************************************************************************  
// *************************************************************************  
void OrbitalSelect::DrawIcons()  
{  
  
    int	i, j, iwf, miwf;  
  
    fontname = fonts->menu;
    SetWidgetFont(draw_area, fontname);
    fontheight= FontHeight(fontname);
 
    j= yspace;  
    iwf= 0;  
  
    //**** form grid ****  
    for (n= 1; n<=6; ++n)  
    {  
	i= 1;  
	if (n < 1) i= fontheight;  
        sprintf (lwf[0], "n= %d", n);  
        DrawText(lwf[0], 100, j+i+fontheight);  
	miwf= 0;  
  
	for (l= 0; l<=n-1; ++l)  
        {  
  
            for (m= -l; m<=l; ++m)  
            {  
                iwf++;  
                miwf++;  
                DrawBox (x1->get(iwf),      y1->get(iwf),  
                         deltax->get(iwf),  deltay->get(iwf));  
                i= 0;  
                if (n == 6) i= 15;  
                i= TextWidth (fonts->Smenu, lwf[miwf]);  
		#if DEBUGDRAWBOX  
		    cout << "TextWidth i="<< i<<"\n";  
		#endif  
  
		#if DEBUGSELECTSTR  
		    cout << "lwf["<<miwf<<"]\n";  
		    cout << lwf[miwf] <<"\n";  
		#endif  
                DrawTextH(draw_area, FALSE, lwf[miwf], 
				( x2->get(iwf) + x1->get(iwf) - i )/2,   
				( y1->get(iwf)+fontheight )	);  
            }  
	}  
  
	j=y2->get(iwf)+1;  
        DrawLine (0,j, 639, j);  
    }  
    // prepare for the mouse CB's
    lastcell=0;  
    cell=0;

    return;  
}  
  
// *************************************************************************  
// *************************************************************************  
int OrbitalSelect::SetCell (int x, int y)  
{  
    // works out in which cell the mouse event occured
 
    // **** see where mouse is ****  
    int iwf= 1;  
  
    // loop thru all the boxes until mouse co-ord is found within . . .  
    while ( (iwf<=91) &&  
              (     (x1->get(iwf)>x) || (x2->get(iwf)<x) ||  
                    (y1->get(iwf)>y) || (y2->get(iwf)<y)      )  
          ) iwf++;  
  
    #if DEBUGSELECT  
	cout << "x1, y1 "<< x1->get(iwf) << ", "<< y1->get(iwf) << "\n"  
	     << "x2, y2 "<< x2->get(iwf) << ", "<< y2->get(iwf) << "\n"  
	     << "cell= "<<iwf <<"\n";  
    #endif  
  
    lastcell = cell;  
  
    // this next line is a contraction of the following in comment . . .  
    return ( InCell= (TRUE&&(cell= ( ((iwf>0)&&(iwf <92)) ? iwf : 0 )) ) );  
  
}  
  
// *************************************************************************  
// *************************************************************************  
void OrbitalSelect::MarkCell ()  
{
    // hi-lights the current and last box (iff they're different)
  
    //**** highlight box ****  
    #if DEBUGMARKCELL  
	cout << lastcell << " " <<cell<<"\n";  
    #endif  
    if ( (lastcell) && (cell!=lastcell) )  
    {  
         DrawFilledBox (x1->get(lastcell),     y1->get(lastcell),  
                        deltax->get(lastcell), deltay->get(lastcell));  
    }  
    if ((cell!=lastcell)&&(InCell))  
    {  
    DrawFilledBox (x1->get(cell),     y1->get(cell),  
                   deltax->get(cell), deltay->get(cell));  
    }  
    return;  
}  
  
// *************************************************************************  
// *************************************************************************  
void nlmButtonDown (Widget w, int which, int x, int y, void *data)  
{
    // CB for mouse down
  
    if (which != 1) return;  
  
    #if DEBUGNLMCALLBACK  
	cout << "in nlmButtonDown: x,y "<< x<<", "<<y<<"\n";  
    #endif  
  
    MyProgram *me = (MyProgram *)data;  
  
    //returns true iff in cell   
  
    SetDrawMode(SANE_XOR);  
    if (me->mainwin.orbitalselect.SetCell(x, y))  
    {  
        me->mainwin.orbitalselect.MarkCell();  
  
        SetButtonDownCB  (me->mainwin.draw_area, NULL        );  
        SetMouseMotionCB (me->mainwin.draw_area, nlmMouseMove);  
        SetButtonUpCB    (me->mainwin.draw_area, nlmButtonUp );  
    }  
    return;  
}  
  
// *************************************************************************  
// *************************************************************************  
void nlmMouseMove (Widget w, int x, int y, void *data)  
{  
    #if DEBUGNLMCALLBACK  
	cout << "in nlmMouseMove: x,y "<< x<<", "<<y<<"\n";  
    #endif  
  
    MyProgram *me = (MyProgram *)data;  
  
    me->mainwin.orbitalselect.SetCell(x, y);  
    me->mainwin.orbitalselect.MarkCell();  
    return;  
}  
  
// *************************************************************************  
// *************************************************************************  
void nlmButtonUp (Widget w, int which, int x, int y, void *data)  
{  
    #if DEBUGNLMCALLBACK  
	cout << "in nlmButtonUp: x,y "<< x<<", "<<y<<"\n";  
    #endif  
  
    MyProgram *me = (MyProgram *)data;  
    if (me->mainwin.orbitalselect.SetCell(x, y))  
    {  
        //me->mainwin.orbitalselect.MarkCell();  
        me->mainwin.orbitalselect.SetId();  
	SetMouseMotionCB (me->mainwin.draw_area, NULL);  
	SetButtonUpCB    (me->mainwin.draw_area, NULL);  
	SetDrawMode(GXcopy);  
	me->mode = DRAWING_ORBITAL_DENSITY;  
	//  
	Redisplay (w, me->mainwin.width, me->mainwin.height, me);  
    }  
    else  
    {  
	me->mainwin.orbitalselect.MarkCell();  
        SetButtonDownCB  (me->mainwin.draw_area, nlmButtonDown);  
	SetMouseMotionCB (me->mainwin.draw_area, NULL);  
	SetButtonUpCB    (me->mainwin.draw_area, NULL);  
	SetDrawMode(GXcopy);	    //is this necessary???  
    }  
    return;  
}  
  
